import { system, world } from "@minecraft/server";
import { Common } from "./common";
export class FenceComponent {
    constructor() {
        this.onPlace = this.onPlace.bind(this);
        this.beforeOnPlayerPlace = this.beforeOnPlayerPlace.bind(this);
    }
    onPlace(event) {
        Fence.processNeighbors(event.block);
    }
    beforeOnPlayerPlace(event) {
        Fence.setAsPlaced(event.block);
    }
}
FenceComponent.id = 'bf_rb:fence';
export class Fence {
    static initialize() {
        if (Fence.initialized)
            throw Error('Instance of Fence already active');
        const processBlockAndNeighbors = (block) => {
            if (block.hasTag(Fence.tagId)) {
                Fence.processNeighbors(block);
            }
            else {
                const neighbors = Common.getBlockNeighbors(block);
                for (const neighbor of Common.getNeighborIterator(neighbors)) {
                    if (neighbor.hasTag(Fence.tagId))
                        Fence.processNeighbors(neighbor);
                }
            }
        };
        world.beforeEvents.playerBreakBlock.subscribe((event) => {
            const run = system.run(() => {
                processBlockAndNeighbors(event.block);
                system.clearRun(run);
            });
        });
        world.afterEvents.playerPlaceBlock.subscribe((event) => {
            processBlockAndNeighbors(event.block);
        });
        Fence.initialized = true;
    }
    static processNeighbors(block) {
        const directions = {
            north: Fence.stateId.North,
            south: Fence.stateId.South,
            east: Fence.stateId.East,
            west: Fence.stateId.West,
        };
        const blocks = [block];
        const neighbors = Common.getBlockNeighbors(block);
        // Collect all neighboring blocks of the same type
        for (const direction in neighbors) {
            const neighbor = neighbors[direction];
            if (neighbor && neighbor.hasTag(Fence.tagId)) {
                blocks.push(neighbor);
            }
        }
        // Process each block in the list
        for (const currentBlock of blocks) {
            if (currentBlock.isAir || currentBlock.isLiquid || !currentBlock.isValid())
                continue;
            let permutation = currentBlock.permutation;
            const currentNeighbors = Common.getBlockNeighbors(currentBlock);
            // Apply the states dynamically based on the current block's neighbors
            for (const [direction, stateId] of Object.entries(directions)) {
                if (permutation.getState(stateId) !== undefined) {
                    const isConnected = currentNeighbors[direction]?.isValid();
                    permutation = permutation.withState(stateId, isConnected || false);
                }
            }
            // Apply the final permutation with all the state changes to the current block
            currentBlock.setPermutation(permutation);
        }
    }
    static setAsPlaced(block) {
        system.waitTicks(1).then(() => {
            block.setPermutation(block.permutation.withState('bf:placed_bit', true));
        });
    }
}
Fence.initialized = false;
Fence.tagId = 'fence';
Fence.stateId = {
    North: "bf:connection_type_north",
    South: "bf:connection_type_south",
    East: "bf:connection_type_east",
    West: "bf:connection_type_west"
};
Fence.initialize();
